function [ boolean ] = temp_sal( p_nemo_data, p_config )
%TEMP_SAL Generates a temperature/salinity plot for every float given.
%   Generates a plot with two subplots, depth on time.
%   The colors of the first plot are given by the temperature, the second
%   one shows the salinity.
%
%   Parameters:
%   p_nemo_data:        The nemo data.
%   p_config:           The nemo configuration.
%
%   Returns:
%   boolean:            False on fail.

%% Initialize return variable
boolean = false;

%% Parameter check %TODO

%% Get config parameter
% get plot details
width = p_config.PLOT_TEMP_SAL.image_width;
height = p_config.PLOT_TEMP_SAL.image_height;
paperunit = p_config.PLOT_TEMP_SAL.image_paperunit;
image_type = p_config.PLOT_TEMP_SAL.image_type;
directory = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_TEMP_SAL.storage_directory_image);
directory_fig = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_TEMP_SAL.storage_directory_fig);
ylabel_fontsize = p_config.PLOT_TEMP_SAL.ylabel_fontsize;
xlabel_fontsize = p_config.PLOT_TEMP_SAL.ylabel_fontsize;
axis_fontsize = p_config.PLOT_TEMP_SAL.axis_fontsize;
title_fontsize = p_config.PLOT_TEMP_SAL.title_fontsize;

if (~isdir(directory))
    mkdir(directory);
end
if ~exist(directory_fig, 'dir')
    mkdir(directory_fig);
end

%% Collect required data

switch (class(p_nemo_data{1}))
    case 'struct' % only one float is to be processed
        boolean = plot_float(p_nemo_data);
    case 'cell' % more than one float has to be processed
        boolean = all(cellfun(@plot_float, p_nemo_data));
    otherwise
        disp([mfilename ': Class ' class(p_nemo_data{1}) ' not supported for plotting!']);
end

    function boolean = plot_float(p_float)
        boolean = false;
        profiles_data = nemo.float.collect(p_float, 'PROFILE_DATA', '');
        profiles_data_combined = nemo.float.collect(p_float, 'PROFILE_DATA', '',  'combine');
        max_pressure = max(profiles_data_combined(:, 7));
        profiles_datetime = nemo.float.collect(p_float, ...
                                               'SECOND_ORDER_INFORMATION', ...
                                               'best_guess_datevec');
        profiles_datetime = datenum(profiles_datetime(:, 1:6));
        if (isempty(profiles_data))
            boolean = true;
            return
        end
        
        % initialize figure
        figure_id = figure('Visible', 'Off');
        set(figure_id,'InvertHardcopy','on');
        set(figure_id,'PaperUnits', paperunit, 'Position', [ 0 0 width height]);
        papersize = get(figure_id, 'PaperSize');
        left = (papersize(1) - width)/2;
        bottom = (papersize(2) - height)/2;
        myfiguresize = [left, bottom, width, height];
        set(figure_id,'PaperPosition', myfiguresize);
        
        % Temperature plot
        sub(1) = subplot(2, 1, 1);
        hold on
        box on
        grid on
        set(gca, 'YLimMode', 'manual', 'YDir', 'reverse', 'YLim', [0 max_pressure]);
        set(gca, 'fontsize', axis_fontsize)
        ylabel('Depth [m]', 'FontSize', ylabel_fontsize);
        % get required information about x-axis limits and ticks
        x_diff = max(profiles_datetime) - min(profiles_datetime);
        set(gca, 'XLimMode', 'auto');
        if (x_diff < 30)
            set(gca, 'XTick', [min(profiles_datetime):5:max(profiles_datetime)]);
        elseif (x_diff < 365)
            set(gca, 'XTick', [min(profiles_datetime):30:max(profiles_datetime)]);
        else
            set(gca, 'XTick', [min(profiles_datetime):365:max(profiles_datetime)]);
        end
        time_combined = [];
        for i = 1:size(profiles_datetime, 1)
            temperature = profiles_data{i}(:, 5);
            pressure = profiles_data{i}(:, 7);
            % erase all invalid values
            valid_points = (pressure >= 0) & (pressure <= 2000) & (temperature < 10);
            temperature = temperature(valid_points);
            pressure = pressure(valid_points);
            time = repmat(profiles_datetime(i, :), size(pressure, 1), 1);
            time_combined = [time_combined; time];
            scatter(time, pressure, 45, temperature, 'filled');
        end
        datetick('x','yyyy-mm-dd', 'keepticks', 'keeplimits');
%         xlabel('Date of profile', 'FontSize', xlabel_fontsize);


        c = colorbar;
        cmocean('thermal');
        %colormap(gca, 'jet');
        %caxis([-5, 15]);
        c.Label.String = ['Temperature [' char(176) 'C]'];
        clear c
        % create main title
        if (p_float{1}.SECOND_ORDER_INFORMATION.best_guess_position_lat > 0)
            deployment_string = ' (Arctic)';
        else
            deployment_string = ' (Antarctica)';
        end
        t = title(...
             [ 'Float ' ...
               num2str(p_float{1}.FLOAT_IDENTIFICATION.float_serial_number) ...
               ' @ ' ...
               datestr(datetime('now', 'Format', 'dd-MMM-yyyy HH:mm:SS')) ...
               deployment_string ...
             ]);
        set(t, 'FontSize', title_fontsize);
        clear t
        hold off
        
        % Salinity plot
        sub(2) = subplot(2, 1, 2);
        hold on
        box on
        grid on
        set(gca, 'YLimMode', 'manual', 'YDir', 'reverse', 'YLim', [0 max_pressure]);
        set(gca, 'fontsize', axis_fontsize)
        ylabel('Depth [m]', 'FontSize', ylabel_fontsize);
        xlabel('Date of profile', 'FontSize', xlabel_fontsize);
        
        % get required information about x-axis limits and ticks
        x_diff = max(profiles_datetime) - min(profiles_datetime);
        set(gca, 'XLimMode', 'auto');
        if (x_diff < 30)
            set(gca, 'XTick', [min(profiles_datetime):5:max(profiles_datetime)]);
        elseif (x_diff < 365)
            set(gca, 'XTick', [min(profiles_datetime):30:max(profiles_datetime)]);
        else
            set(gca, 'XTick', [min(profiles_datetime):365:max(profiles_datetime)]);
        end
        for i = 1:size(profiles_datetime, 1)
            salinity = profiles_data{i}(:, 9);
            pressure = profiles_data{i}(:, 7);
            % erase all invalid values
            valid_points = (pressure >= 0) & (pressure <= 2000) & (salinity > 10);
            time = repmat(profiles_datetime(i, :), size(pressure(valid_points), 1), 1);
            scatter(time, ...
                    pressure(valid_points), 45, salinity(valid_points), 'filled');
        end
        datetick('x','yyyy-mm-dd', 'keepticks', 'keeplimits');
        c = colorbar;
        cmocean('haline');
        %colormap(gca, [0:0.001:1; repmat(0, 1, 1001); fliplr(0:0.001:1)]');
        %caxis([33, 36]);
        c.Label.String = 'Salinity';
        hold off

         
        set(gcf, 'Renderer', 'painters');
        print(figure_id, ...
         [directory p_config.PLOT_TEMP_SAL.image_filename ...
           num2str(p_float{1}.FLOAT_IDENTIFICATION.float_serial_number)] ...
         , image_type);
     
        if (p_config.PLOT_TEMP_SAL.save_figure)
            set(gcf,'CreateFcn','set(gcf,''Visible'',''on'')');
            saveas(figure_id, ...
                [directory_fig ...
                 p_config.PLOT_TEMP_SAL.figure_filename ...
                 num2str(p_float{1}.FLOAT_IDENTIFICATION.float_serial_number)]);
        end
        
        close(figure_id);
     
        clear figure_id f
        boolean = true;
        
    end

end

