function [ fixed_nemo_data ] = estimate_unknowns( p_nemo_data, p_estimation_method )
%ESTIMATE_UNKNOWN_POSITIONS Calculates the "best guess" profile positions.
%   WARNING: Do not use this function alone! If you want to have consistent
%   position data, please use
%   nemo.postprocessing.position.create_best_guess.
%
%   This function iterates over all nemo floats, given in the parameter
%   p_nemo_data, determining the missing GPS information of profiles where
%   the float did not come to the water surface. This is done via gcwaypts 
%   (Mapping Toolbox) between the known GPS/Iridium positions. They have been
%   stored in the SECOND_ORDER_INFORMATION struct by
%   nemo.postprocessing.position.estimate_best_guess().
%   If p_nemo_data is a string, it reads the profiles with the
%   read_profiles function. So be sure that you added read_profiles.m to
%   your search path!
%
%   Returns: p_nemo_data with interpolated positions, false on fail.
%
%   Parameters:
%   p_nemo_data:    Either has to be a multidimensional struct (created by
%                   nemo.load.data()) or a string, where the nemo float data
%                   is stored.


%% Initialize return variables
fixed_nemo_data = false; % default value if function fails

%% Check passed parameter and read profiles if necessary
parameter_error = false; % indicates if a parameter error was raised

if (ischar(p_nemo_data)) % read profiles using read_profiles.m
    if (~isdir(p_nemo_data)) % parameter is not a directory
        parameter_error = true;
    else % read profiles
        fixed_nemo_data = nemo.load.data(p_nemo_data);
    end
else if (~iscell(p_nemo_data)) % raise error
        parameter_error = true;
    else
        fixed_nemo_data = p_nemo_data; % store nemo data in return variable
    end
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please provide either a ' ...
        'directory path or a struct created by read_profiles.m!']);
    return;
else % do some cleanup
    clear parameter_error;
end

% If estimation method nan is chosen, do nothing because there is already
% NaN given in the unknown positions by
% nemo.postprocessing.position.estimate_best_guess().
if (strcmp(p_estimation_method, 'nan'))
    return
end

%% Position estimation

for o_float_index = 1:length(fixed_nemo_data)
    
    current_float = fixed_nemo_data{o_float_index}; % stores current float in tmp var
    profile_count = length(current_float);
    
    
    % find all pairs that have incomplete positions in between
    
    o_profile_index = 1;
    
    while (o_profile_index <= profile_count) % process every profile
        if (~isfield(current_float{o_profile_index}, 'SECOND_ORDER_INFORMATION') ...
                || ~isfield(current_float{o_profile_index}.SECOND_ORDER_INFORMATION, 'best_guess_position_lat') ...
                || ~isfield(current_float{o_profile_index}.SECOND_ORDER_INFORMATION, 'best_guess_position_lon') ...
                || ~isfield(current_float{o_profile_index}.SECOND_ORDER_INFORMATION, 'best_guess_position_src')) % security check
            disp([mfilename ': No best guess position available in float ' num2str(current_float{o_profile_index}.FLOAT_IDENTIFICATION.float_serial_number) ...
                ' profile ' num2str(current_float{o_profile_index}.PROFILE_TECHNICAL_DATA.xmit_profile_number) '! Skipping...']);
            o_profile_index = o_profile_index + 1;
            continue
        end
        
        % check for NaN, if it is the first index, the deployment position
        % will be the first position for the interpolation
        index_last_position = o_profile_index;
        while (o_profile_index <= profile_count) ...
                && isnan(current_float{o_profile_index}.SECOND_ORDER_INFORMATION.best_guess_position_lat)
            o_profile_index = o_profile_index + 1;
        end
        index_next_position = o_profile_index;
        
        if (index_last_position == index_next_position) % no gap has been found
            % increase iteration var and continue
            o_profile_index = o_profile_index + 1;
            continue;
        else
            % a gap has been found, so the last position index is wrong!
            index_last_position = index_last_position - 1;
        end
        
        % a gap has been found
        if (index_last_position == 0) % use deployment position as last position
            last_position.lat = current_float{1}.DEPLOYMENT_INFO.deployment_position(1);
            last_position.lon = current_float{1}.DEPLOYMENT_INFO.deployment_position(2);
            last_position.src = 'deployment';
            last_position.datetime = 0;
        else
            last_position.lat = current_float{index_last_position}.SECOND_ORDER_INFORMATION.best_guess_position_lat;
            last_position.lon = current_float{index_last_position}.SECOND_ORDER_INFORMATION.best_guess_position_lon;
            last_position.src = current_float{index_last_position}.SECOND_ORDER_INFORMATION.best_guess_position_src;
            last_position.datetime = current_float{index_last_position}.PROFILE_TECHNICAL_DATA.xmit_surface_start_time;
        end
        
        % if there is no position at all of the float, do not create best
        % guess for it
        if index_next_position > profile_count
            break;
        end
        next_position.lat = current_float{index_next_position}.SECOND_ORDER_INFORMATION.best_guess_position_lat;
        next_position.lon = current_float{index_next_position}.SECOND_ORDER_INFORMATION.best_guess_position_lon;
        next_position.src = current_float{index_next_position}.SECOND_ORDER_INFORMATION.best_guess_position_src;
        next_position.datetime = current_float{index_next_position}.PROFILE_TECHNICAL_DATA.xmit_surface_start_time;
        
        intermediate_profile_count = index_next_position - index_last_position;
        
        switch p_estimation_method
            case 'position based'
                position_based_estimation
            case 'time based'
                time_based_estimation
            otherwise
                warning([mfilename ': Estimation method not available. Using position based instead...']);
                position_based_estimation
        end
        
        
        
    end
    
    % store the corrected profiles into struct that will be returned
    fixed_nemo_data{o_float_index} = current_float;
    
    
end

    function position_based_estimation
        % calculate intermediate points
        [intermediate_points_lat, intermediate_points_lon] = gcwaypts( ...
            last_position.lat, ...
            last_position.lon, ...
            next_position.lat, ...
            next_position.lon, ...
            intermediate_profile_count ... % function requires legs, not points
            );
        
        % store all positions in best guess variable
        for i = 1:intermediate_profile_count
            current_float{index_last_position + i}.SECOND_ORDER_INFORMATION.best_guess_position_lat = intermediate_points_lat(i + 1);
            current_float{index_last_position + i}.SECOND_ORDER_INFORMATION.best_guess_position_lon = intermediate_points_lon(i + 1);
            current_float{index_last_position + i}.SECOND_ORDER_INFORMATION.best_guess_position_src = [ last_position.src '_gcwaypts_' next_position.src ];
        end
    end

    function time_based_estimation
        difference_in_seconds = next_position.datetime - last_position.datetime;
        
        [intermediate_points_lat, intermediate_points_lon] = gcwaypts( ...
            last_position.lat, ...
            last_position.lon, ...
            next_position.lat, ...
            next_position.lon, ...
            difference_in_seconds + 1 ... % function requires legs, not points
            );
        
        for i = 1:intermediate_profile_count-1
            profile_datetime = current_float{index_last_position + i}.PROFILE_TECHNICAL_DATA.xmit_surface_start_time;
            profile_to_last_position = profile_datetime - last_position.datetime;
            current_float{index_last_position + i}.SECOND_ORDER_INFORMATION.best_guess_position_lat = intermediate_points_lat(profile_to_last_position);
            current_float{index_last_position + i}.SECOND_ORDER_INFORMATION.best_guess_position_lon = intermediate_points_lon(profile_to_last_position);
            current_float{index_last_position + i}.SECOND_ORDER_INFORMATION.best_guess_position_src = [ last_position.src '_gcwaypts_' next_position.src ];
        end
        
    end

end % estimate_unknowns

